/**
 * SSLPressure 
 * Author: James Dickson.
 * License: GPL v3
 */
#include <stdio.h>
#include <winsock2.h>
#include <time.h>
#include "ssl2functions.h"
#include "wsocket.h"

void ssl2CreateHelloMessageEx(SSL_CLIENT_HELLO_START *msg, WORD ssl_version, BYTE *cipherSpecs, WORD cipherSpecLength)
{
	WORD helloLength = 25 + cipherSpecLength;

	// RECORD
	msg->sslRecord.length = (0x8000) | helloLength;

	msg->sslRecord.handshakeMessageType = SSL_CONTENT_TYPE_HANDSHAKE;	
	msg->sslRecord.version = ssl_version;
	msg->sslRecord.cipherSpecLength = cipherSpecLength; // 3 bytes for every cipher
	msg->sslRecord.sessionIDLength = 0x0000;
	msg->sslRecord.challengeLength = 16;
	

	// Ciphers
	msg->cipherSpecs = cipherSpecs;

	// Challenge (should really be random .... but not important for this app).
	msg->challenge[0] = 's';
	msg->challenge[1] = 's';
	msg->challenge[2] = 'l';
	msg->challenge[3] = 'p';
	msg->challenge[4] = 'r';
	msg->challenge[5] = 'e';
	msg->challenge[6] = 's';
	msg->challenge[7] = 's';
	msg->challenge[8] = 'u';
	msg->challenge[9] = 'r';
	msg->challenge[10] = 'e';
	msg->challenge[11] = '1';
	msg->challenge[12] = '2';
	msg->challenge[13] = '3';
	msg->challenge[14] = '4';
	msg->challenge[15] = '5';
}


/**
 * SSL2 builds (in this app) on TLS-protocol (as opposed to real world).
 */
BOOL ssl2TestConnection(
						   //WORD tls_version_not_used_here, 
						   WORD *cipher, 
						   TLS_EXTENSION *extensions, 
						   WORD extensionLength, 
						   DWORD cipherCount,
						   SOCKADDR_IN lSockAddr,
						   int secondTimeout,
						   WORD currentSSLVersion, 
						   WORD *foundCipher, 
						   DWORD dwCipher,
						   DWORD *dwFoundCipher)
{
	SOCKET lhSocket = 0;

	// Connect to server...
	int bConnected = getConnection(&lhSocket, &lSockAddr, secondTimeout);
	if(! bConnected)
	{
		return FALSE;
	}

	// Create the cipher specification.
	BYTE *cipherSpecs = new BYTE[cipherCount * 3 + 1];
	int cPos = 0;

	// We assume that the cipher is specified using 16-bit format. So we convert to 24-bit.
	for(int i=0; i < (cipherCount * 3); i+=3)
	{
		cipherSpecs[i] = HIBYTE(cipher[cPos]);
		cipherSpecs[i+1] = 0x00;
		cipherSpecs[i+2] = LOBYTE(cipher[cPos]);

		cPos++;
	}

	// Create SSL2-hello.
	SSL_CLIENT_HELLO_START msg1;	
	ssl2CreateHelloMessageEx(&msg1, SSL_VERSION_2, cipherSpecs, cipherCount * 3);
	ssl2SendClientHelloMessage(&msg1, lhSocket, 25 + (cipherCount * 3));

	// Receive server-hello
	unsigned char sockBuf[8192];
	int rsize = recv(lhSocket, (char *) sockBuf, 8192, 0);

	// Destroy connection
	closesocket(lhSocket);

	// Found at least one valid, so we go through the whole block
	if(rsize > 44 && sockBuf[2] == SSL2_HANDSHAKE_TYPE_SERVER_HELLO)
	{
		TLS_RECORD tlsRecord;				
		tlsGetHeader((BYTE*) sockBuf, &tlsRecord);

		DWORD certLength = sockBuf[7];
		certLength <<=  8;
		certLength += sockBuf[8];

		DWORD cipherSpecLength = sockBuf[9];
		cipherSpecLength = (cipherSpecLength << 8) + (sockBuf[10]);

		// No ciphers found.
		if(cipherSpecLength <= 0)
		{
			return FALSE;
		}


		int idLen = 12 + certLength;

		// Get the found cipher.
		if((idLen + 3) < rsize)
		{			
			DWORD dwTemp = (sockBuf[idLen + 1] << 16) | (sockBuf[idLen + 2]  << 8) | sockBuf[idLen + 3];
			*dwFoundCipher = dwTemp;
		}

		return TRUE;
	}
	return FALSE;
}




/**
 * Send TLS-ClientHello to server
 */
int ssl2SendClientHelloMessage(SSL_CLIENT_HELLO_START *msg, SOCKET lhSocket, int rawLength)
{
	int len = rawLength + 2;
	unsigned char *buf = (unsigned char *) malloc(len); 
	int pos = 0;

	buf[pos++] = (BYTE) HIBYTE(msg->sslRecord.length);
	buf[pos++] = (BYTE) LOBYTE(msg->sslRecord.length);
	buf[pos++] = (BYTE) msg->sslRecord.handshakeMessageType;
	buf[pos++] = (BYTE) HIBYTE(msg->sslRecord.version);
	buf[pos++] = (BYTE) LOBYTE(msg->sslRecord.version);
	buf[pos++] = (BYTE) HIBYTE(msg->sslRecord.cipherSpecLength);
	buf[pos++] = (BYTE) LOBYTE(msg->sslRecord.cipherSpecLength);

	buf[pos++] = (BYTE) HIBYTE(msg->sslRecord.sessionIDLength);
	buf[pos++] = (BYTE) LOBYTE(msg->sslRecord.sessionIDLength);

	buf[pos++] = (BYTE) HIBYTE(msg->sslRecord.challengeLength);
	buf[pos++] = (BYTE) LOBYTE(msg->sslRecord.challengeLength);

	// Ciphers ...Instead of memcpy
	for(int x=0; x < msg->sslRecord.cipherSpecLength; x++)
	{
		buf[pos++] = msg->cipherSpecs[x];
	}

	// Challenge...
	for(int x=0; x < msg->sslRecord.challengeLength; x++)
	{
		buf[pos++] = msg->challenge[x];
	}


	// Send hello to server...
	int retval = send(lhSocket,(char *) buf, len, 0);

	// Cleanup.
	free(buf);

	return retval;
}

